/*
   Armadillo's spawner
   1999-2000 Randall Maas
*/

#include "dillib.h"

/* @txh @txhcat=Processes */
pid_t __Dillo_spawn(int* filedes, char* const ArgV[], char* const env[],
		  unsigned Flags)
{
   /*
      Description
       This spawns and executes a program.  @var{Name} is the path of the
       program.  @var{env} is the environment that the program should use.

       If the @code{DILLO_NEWIN} flag is set, @var{FileDes[1]}
       can be used to to send data to the child.  Otherwise @var{FileDes[1]}
       is set to a meaningless value.

       If the @code{DILLO_SHARE_ERR} flag is set, the child's standard error
       is dup'd to its standard output.  Otherwise, its standard error
       channel is the same as its parent.

       If the @code{DILLO_TTY} flag is set, the child will be set to
       communicate using a master/slave pseudo-tty instead of standard pipe.


      Return Value
       -1 on error, otherwise the process id of the spawned task.

      @end txh
    */ 
   pid_t child_pid;

   if ((child_pid = __Dillo_fork2(filedes, Flags)) < 0)
     {
        /* The fork procedure failed, and it spit out an error message,
	   so we will return to the caller.
	*/ 
	return -1;
     }

   if (child_pid)
     {
        /* We are the parent. */
	return filedes[0];
     }
 
   /* this is the child */
   execve (ArgV[0], ArgV, env);

   /* Returns on error only */
   Err_cmdline(DILLO_EXEC_FAILED, ArgV[0], strerror(errno));

   _exit(-1);
   return -1;
}
