/*
  Psuedo terminal routines for 4.3 BSD
  1993-2000 Randall Maas

  Based upon stuff from glibc manual, Fraser McCrossan, and some text book
  I read a long time ago.
*/

#include "platform.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <stdlib.h>
#ifdef HAVE_STROPTS_H
# include <stropts.h>
#endif

#define PTYCHAR1  "abcdepqrstuvwxyz"
#define Hex_Digit "0123456789abcdef" /* y */

/* @txh @txhcat=Processes */
int __PTY_pipe(int filedes[2])
{
   /*
      Description
       This is like @code{pipe()} except that it opens a
       pair of bidirection pipes using psuedo-tty's.  @var{filedes[0]} is
       the master tty, and @var{filedes[1]} is the slave tty.

      Return Value
       Returns 0 on success otherwise -1 on error

      @end txh
   */

  #if defined(HAVE_GETPT)
  /* Try to use the UNIX98, glibc etc compatible way */
  const char* name;
  if ((filedes[0] = getpt()) < 0) return -1;

  if (
#ifdef HAVE_GRANTPT
      grantpt(filedes[0]) < 0 ||
#endif
#ifdef HAVE_UNLOCKPT
      unlockpt(filedes[0]) < 0 ||
#endif
      !(name=ptsname(filedes[0])) ||
      (filedes[1] = open(name, O_RDWR)) < 0)
    {
       close(filedes[0]);
       return -1;
    }

  if (isastream(filedes[1]) &&
      (ioctl(filedes[1], I_PUSH, "p_tem") < 0 ||
       ioctl(filedes[1], I_PUSH, "ldterm") < 0))
    {
       close(filedes[0]);
       close(filedes[1]);
       return -1;
    }
  return 0;

#else

  const char *Ptr;
  struct stat StatBuff;
  char Pty_Name[]="/dev/ptyXY"; /* The name of the psuedo terminal
				     "/dev/[pt]tyXY] = 10 chars + null byte */

  /*
    Open the master half - "/dev/pty[pqrs][0x0-0xf]".  There is no easy 
    way to obtain an available minor device (similar to a streams clone
    open) -- we have to try them all until we find an unused one.
  */
  
  for (Ptr = PTYCHAR1; *Ptr; Ptr++)
   {
      const char* IPtr;
      Pty_Name[8] = *Ptr; /* X */
      Pty_Name[9] = '0'; /* Y */
      
      /* 
	 If this name, "/dev/ptxX0" doesn't even exist, then we can quit 
	 now.  It means the system doesn't have /dev entries for this group
	 of 16 ptys.
      */
      if (stat(Pty_Name, &StatBuff) < 0) continue;
      
      for (IPtr = Hex_Digit; *IPtr; IPtr++)
       {
	  Pty_Name[9] = *IPtr;
	  if ((filedes[0] = open(Pty_Name, O_RDWR)) < 0)
	    continue;

	  /*
	     Open the slave half of a psuedo-terminal.  Note that the master
	     half of a PTY is a single-open device, so there isn't a race
	     condition between opening the master above and opening the slave
	     below.  The only way the slave open will fail is if someone has
	     opened the slave  without first opening the master.
	  */
	 
	  Pty_Name[5] = 't'; /* Change "/dev/ptyXY" to /dev/ttyXY */
	  if ((filedes[1] = open(Pty_Name, O_RDWR)) < 0)
	    {
	       close(filedes[0]);
	       continue;
	    }

	  /* Both opened */
	  return 0;
       }
   }
  
  return -1; /* Couldn't open master, assume all pty's are in use */
#endif
}
