/*
    AutoHelp file reader.
    Copyright 2000 Randall Maas (randym@acm.org)
    Copyright 1998-2000 Fraser Mc

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "dillib.h"

#define BUFSIZE 1024
#define COMMENT '#' /* comment character in rc files */
#define SAMEAS  "=" /* entries with this get set to the previous equivalent */

static int AppendText(__AutoHelpEntry_t* Ptr, const char* Str);

/* @txh */
int __AutoHelp_load(const char *Path, __AutoHelpEntry_t**Rules, size_t* NRules)
{
   /*
      Description
       Opens the specified file and reads in the table of help information

      Return Value
       -1 on error, 0 on success
      @end txh
   */

  FILE *fp;
  char in[BUFSIZE];
  unsigned int lineno=0;
  char hl[BUFSIZE], regex[BUFSIZE];
  size_t NAllocd=*NRules;

  if ((fp = fopen(Path, "r")) == NULL) {
    if (errno != ENOENT) {
      Err_cmdline("unable to open \"%s\" (%s)\n", Path, strerror(errno));
    }
    return -1;
  }

  while (fgets(in, sizeof(in), fp) != NULL) {
    ++lineno;
    if (in[0] == COMMENT) continue;
    if (sscanf(in, ":%s %[^\n]\n", hl, regex) == 2) {
      /* a control line, make a new help entry */
      __AutoHelpEntry_t *new;
      if (NAllocd <= *NRules)
	{
	  NAllocd = (NAllocd+1)<<1;
	  if (!(*Rules = realloc(*Rules, sizeof(**Rules)*NAllocd)))
	    {
	      fprintf(stderr, "%s:%u hl %s regex %s\n", Path, lineno,
		      hl, regex);
	      return -1;
	    }
	}
      new = *Rules+(*NRules)++;
      new->Hint = NULL;
#if 0
      new->helplevel = 0;
      if (strcmp(hl, SAMEAS) == 0 && current != NULL) {
	new->helplevel = current->helplevel;
      }
      else {
	for (i = hl; *i != '\0'; ++i) {
	  if (*i >= '0' && *i <= '9') {
	    new->helplevel |= 1 << (*i - '0');
	  }
	}
      }
#endif
      if (strcmp(regex, SAMEAS) == 0 && *NRules) {
	new->Reg = new[-1].Reg;
      }
      else if (regcomp(&(new->Reg), regex,
		       REG_EXTENDED|REG_NOSUB) != 0) {
	Err_cmdline("%s:%u: bad regular expression \"%s\"\n", Path, lineno,
		    regex);
      }
      continue;
    }

    /* not a control line, must be a help text line */
    if (!*NRules) {
      Err_cmdline("%s:%u: help text without control line\n", Path, lineno);
      continue;
    }

    if (AppendText(*Rules + *NRules -1, in)) return -1;
  }

  fclose(fp);
  return 0;
}

static int AppendText(__AutoHelpEntry_t* Ptr, const char* Str)
{
   /*
      Description
      Return Value
       0 on success, otherwise error
   */
   char *newhint;
   if (Ptr->Hint == NULL) { /* nothing yet */
     if ((Ptr->Hint = (char *) malloc(strlen(Str)+1)) == NULL)
       return -1;
     strcpy(Ptr->Hint, Str);
     return 0;
   }

   /* stuff exists, append to it */
   if ((newhint = (char *) realloc(Ptr->Hint,
				   strlen(Ptr->Hint) +
				   strlen(Str) + 1)) == NULL)
     return -1;
   strcpy(newhint + strlen(newhint), Str);
   Ptr->Hint = newhint;
   return 0;
}
