/*
   Automatic Help Window access routines
   Copyright 2000 Randall Maas (randym@acm.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "dillib.h"
static __AutoHelpEntry_t* _Help_match(__AutoHelpEntry_t* first,
				      __AutoHelpEntry_t* last,
				      const char* Text);

/* @txh */
void __AutoHelp_init(__AutoHelp_t* VPtr, __AutoHelpEntry_t* Rules, int NRules,
		     LabelProc_t DispProc, void* DispProcData)
{
   /*
      Description
       Fills in the structure.
   */
   VPtr->Rules      = Rules;
   VPtr->NRules     = NRules;
   VPtr->CurrentPrompt= NULL;
   VPtr->DispProc     = DispProc;
   VPtr->DispProcData = DispProcData;
}

void __AutoHelp_text(__AutoHelp_t* VPtr, const char* Text)
{
   /*
      Description
       Given the text, finds a matching helpful hint and displays it (if
       possible).

      @end txh
   */

   __AutoHelpEntry_t *I,
     *End   = VPtr->Rules+VPtr->NRules,
     *QLast = VPtr->CurrentPrompt?VPtr->CurrentPrompt:End;

   /* See if there are any matching rules after or before the current rule
    */
   if (!(VPtr->CurrentPrompt &&
	 (I = _Help_match(VPtr->CurrentPrompt, End, Text))) &&
       !(I = _Help_match(VPtr->Rules, QLast, Text)))
     return;

   /* Print a message keeping the prompt intact */
   if (I->Hint)
     VPtr->DispProc(VPtr->DispProcData, I->Hint /* Short*/, I->Hint /*Long */);
   //       if (I->DoX) Do it;


   /* Make note of the current rule that matched */
   VPtr->CurrentPrompt = I;
}

static __AutoHelpEntry_t* _Help_match(__AutoHelpEntry_t* first,
				      __AutoHelpEntry_t* last,
				      const char* Text)
{
   /*
      Description

      Return Value
       Pointer to the entry that matches, otherwise NULL if no match
    */

   __AutoHelpEntry_t* I;
   for (I = first; I != last; I++)
    {
       if (regexec(&(I->Reg), Text, 0, NULL, 0) == 0)
	 return I; /* This is the one! */
    }
  return NULL;
}
