/*
  Time value difference
  Adapted from the GNU libc manual
*/
#include <sys/time.h>

int timeval_diff(struct timeval* Result, const struct timeval* A,
		 const struct timeval* B)
{
   /*
      Description
       Subtract the timeval values @var{A} and @var{B}, storing the result
       (in @var{Result}).

      Return Value
        Return -1 if the difference is negative, otherwise 0.
   */

   struct timeval C = *B;
  
   /* Perform the carry for the later subtraction. */
   if (A->tv_usec < C.tv_usec)
     {
       int nsec = (C.tv_usec - A->tv_usec) / 1000000 + 1;
       C.tv_usec -= 1000000 * nsec;
       C.tv_sec  += nsec;
     }
   if (A->tv_usec - C.tv_usec > 1000000)
     {
       int nsec = (A->tv_usec - C.tv_usec) / 1000000;
       C.tv_usec += 1000000 * nsec;
       C.tv_sec  -= nsec;
     }
     
   /* Compute the time remaining to wait.
      `tv_usec' is certainly positive. */
   Result->tv_sec = A->tv_sec - C.tv_sec;
   Result->tv_usec= A->tv_usec- C.tv_usec;
   
   /* Return 1 if result is negative. */
   return A->tv_sec < C.tv_sec?-1:0;
}
