/*
   Curses support routines that can be used by the Virtual TTY.
   Copyright 2000 Randall Maas (randym@acm.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
#ifndef __vtcurses_h
# define __vtcurses_h
# ifndef __Platform_h
#  include "platform.h"
# endif
# include <curses.h>
# include <panel.h>

typedef struct
{
  WINDOW* Win;
  PANEL*  Panel;
} __VT_label_t;

typedef struct
{
   int    Visibility; /* Cursor visibility:
			   0 = invisible, 1= normal, 2=very visible */
   attr_t Attr;
   short  Pair;
   int x,y;	    /* Location */
   int Top, Bottom; /* Scroll region */
} __VT_cstate_t;

#define __VC_cattrset(Win,C)   wattrset((WINDOW*)Win,C)
#define __VC_cattron(Win,C)    wattron ((WINDOW*)Win,C)
#define __VC_cattroff(Win,C)   wattroff((WINDOW*)Win,C)
#define __VT_eraseline(SPtr) do{wdeleteln(SPtr); winsertln(SPtr);}while(0)


#define NEW_LINE 0
#define LINE_FEED 1
extern int LineMode;

extern void	__VT_reset(void* SPtr);
void __VC_writen   (void* SPtr, const char* Str, size_t N);
extern int	__VC_color    (void* SPtr, short fc, short bc);
void __VT_dmove    (void* SPtr, int dy, int dx);
extern void __VT_init (void *SPtr) ATTR(INIT);
extern void __VT_clreol(void *SPtr);
extern void __VT_backspace(void* SPtr, unsigned n);
extern void __VT_ins      (void* SPtr, unsigned n);
#define __VT_nextpage(SPtr,n) __VT_scrollpage(SPtr,n)
#define __VT_prevpage(SPtr,n) __VT_scrollpage(SPtr,-n)
extern void          __VT_scrollpage   (void* SPtr, int n);
extern void	     __VT_scrollreset  (void* SPtr);
extern __VT_label_t* __VT_label_new    (const char* FirstStr);
extern void          __VT_label_destroy(__VT_label_t* Ptr);
extern void          __VT_label_text   (__VT_label_t* VPtr, const char *s);
extern void	     __VT_cursave      (void* SPtr);
extern void	     __VT_currest      (void* SPtr);
extern void	     __VT_scrreg	(void* SPtr,int top,int bot);
extern void	     __VT_chide		(void* SPtr);
extern void	     __VT_cshow		(void* SPtr);
extern void	     __VT_100setmode	(void* SPtr, int mode);
extern void	     __VT_100clearmode	(void* SPtr, int mode);

#ifndef def___VT_dmove
EXTERN
#endif
void __VT_dmove(void* SPtr, int dy, int dx)
{
  int y,x, Rows,Cols;
  WINDOW* WPtr = (WINDOW*)SPtr; 
  getmaxyx(WPtr, Rows,Cols);
  getyx (WPtr,y,x);
  y += dy;
  x += dx;
       if (y < 0) {wscrl(WPtr, y); y=0;}
  else if (y >= Rows)
    {wscrl(WPtr,y+1-Rows); y = Rows-1;}
  if (x < 0) x=0;
  wmove (WPtr, y,x);
}

#ifndef def___VT_mcol
EXTERN
#endif
void __VT_mcol(void* SPtr, int Col)
{
  int y,x;
  WINDOW* WPtr = (WINDOW*)SPtr; 
  getyx (WPtr, y,x);
  wmove (WPtr, y,Col);
}

#ifndef def___VT_mrow
EXTERN
#endif
void __VT_mrow(void* SPtr, int Row)
{
  int y,x;
  WINDOW* WPtr = (WINDOW*)SPtr; 
  getyx (WPtr, y,  x);
  wmove (WPtr, Row,x);
}

#ifndef def___VT_nextline
EXTERN
#endif
void __VT_nextline(void *SPtr)
{
   /*
      Description
       This moves the cursor to the start of the next line.  If the next line
       is off the display, the screen will be scrolled.  Otherwise the screen
       contents are not modified.
   */

   int y, x, Rows, Cols;
   
   getmaxyx ((WINDOW*)SPtr, Rows,Cols);
   getyx    ((WINDOW*)SPtr, y,   x);
   if (y+1 < Rows) {wmove(SPtr,y+1,0); return;}
   wscrl((WINDOW*)SPtr, 1);
   wmove(SPtr, Rows-1,0);
}



#ifndef def___VT_scrollpage
EXTERN
#endif
void __VT_scrollpage(void* SPtr, int n)
{
  int Rows, Cols;
   getmaxyx((WINDOW*)SPtr, Rows,Cols);
   wscrl((WINDOW*)SPtr, n* Rows);
}
#endif
