/*
   Curses based label part
   Copyright 2000 Randall Maas (randym@acm.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "vtcurses.h"
#include <ctype.h>
#include <malloc.h>
extern PANEL*  ShellPanel;

/* @txh */
__VT_label_t* __VT_label_new(const char* FirstStr)
{
   /*
      Description
       If @var{FirstStr} is defined, it will be printed to the label.

      Return Value
       NULL on error, otherwise a pointer to the label item
   */

   __VT_label_t* Ptr = malloc(sizeof(*Ptr));
   if (!Ptr) return NULL;

   Ptr->Win  = newwin(3, COLS, LINES-3, 0);
   Ptr->Panel= new_panel(Ptr->Win);

   __VT_init(Ptr->Win);
   wbkgdset(Ptr->Win, ' '| COLOR_PAIR(__VC_color(Ptr->Win,COLOR_WHITE,COLOR_BLUE)));
   werase(Ptr->Win);
   if (FirstStr) __VT_label_text(Ptr, FirstStr);
   return Ptr;
}

void __VT_label_destroy(__VT_label_t* Ptr)
{
   /*
      Description
       Frees all of @var{Ptr}'s resources.
   */

   if (!Ptr) return;
   del_panel(Ptr->Panel);
   delwin   (Ptr->Win);
   free(Ptr);
}

void __VT_label_text(__VT_label_t* VPtr, const char *s)
{
   /*
      Description
       Prints a message to the label pointed to by @var{VPtr}.
      
      @end txh
   */

  const char* Ptr=s;
  int Bold=0;
  int Col= __VC_color(VPtr->Win,COLOR_WHITE,COLOR_BLUE);

  werase(VPtr->Win);
  while (*Ptr)
    {
       size_t Sz = strcspn(Ptr, "+-[<>]");
       if (Sz)
	 {
	    waddnstr(VPtr->Win, Ptr, Sz);
	    Ptr += Sz;
	 }
       switch (*Ptr)
       {
	 /* End the highlight of the command names and variables */
         case ']':
         case '>':
	   if (!Bold)
	     {
	       waddch(VPtr->Win, *Ptr++);
	       break;
	     }
	   Bold--;
	   Ptr++;
	   wattroff(VPtr->Win, A_BOLD);
	   __VC_color(VPtr->Win,COLOR_WHITE, COLOR_BLUE);
	   break;

	   /* Highlight the command names and variables */
         case '[':
         case '<':
	   if (!isalnum(Ptr[1]))
	     {
	       waddch(VPtr->Win, *Ptr++);
	       break;
	     }
	   Bold++;
	   wattron(VPtr->Win, A_BOLD);
	   if (*Ptr=='<') __VC_color(VPtr->Win, COLOR_GREEN, COLOR_BLUE);
	   Ptr++;
	   break;

         case '-':
         case '+':
	   /* Highlight the command options */
	   if (Ptr-1 != s && (isspace(Ptr[-1]) || Ptr[-1] == '\n') &&
	       (Sz = strcspn(Ptr, " ")) && !(Ptr[1]=='-' && Sz==2))
	     {
	       wattron   (VPtr->Win, A_BOLD);
	       waddnstr  (VPtr->Win, Ptr, Sz);
	       Ptr += Sz;
	       wattroff  (VPtr->Win, A_BOLD);
	       continue;
	     }

	   waddch(VPtr->Win, *Ptr++);
	   break;
       }
    }
   Disp_sched();
   show_panel(ShellPanel);
}
