/*
    IO Request Structures
    Randall Maas
    1999
*/

#ifndef __IOReq_h
# define __IOReq_h
# ifndef __Platform_h
#  include "platform.h"
# endif
# ifdef HAVE_SEARCH_H
#  include <search.h>
# endif
# if defined(HAVE_SYS_UIO_H) && HAVE_SYS_UIO_H
#   include <sys/uio.h>
# else
struct iovec {void* iov_base; size_t iov_len;};
static inline ssize_t readv(int fd, const struct iovec* V, int count)
{
   ssize_t Size=0;
   for (; count;count--,V++)
    {
       ssize_t S= read(fd,V->iov_base,V->iov_len);
       if (S <= 0) break;
       Size+=S;
    }
   return Size;
}
static inline ssize_t writev(int fd, const struct iovec* V, int count)
{
   ssize_t Size=0;
   for (; count;count--,V++)
    {
       ssize_t S= write(fd,V->iov_base,V->iov_len);
       if (S <= 0) break;
       Size+=S;
    }
   return Size;
}
# endif

typedef struct __IO_req_t
{
   struct __IO_req_t* q_forw, *q_back;
   int     Op; /*Read(0)/Write(1) */
   ssize_t Status;
   size_t  Count;
   struct iovec* IOReq;
} __IO_req_t;

extern ssize_t IO_Do(int FD, __IO_req_t* IPtr);
extern void FD_bkgnd(int FD);

#ifndef def_FD_bkgnd
extern __inline__
#endif
/* @txh @txhcat=IO */
void FD_bkgnd(int FD)
{
   /*
      Description
       Set the file descriptor to be nonblocking and close-on-exec
   */
   fcntl (FD, F_SETFL, O_NONBLOCK | fcntl(FD, F_GETFL));
   fcntl (FD, F_SETFD, FD_CLOEXEC | fcntl(FD, F_GETFD));
}

#endif
