#ifndef __Dillo_lib_h
# define __Dillo_lib_h

# ifndef __Platform_h
#  include "platform.h"
# endif
# ifndef __Dillo_Msgs_h
#  include "dillomsgs.h"
# endif
# ifdef HAVE_LTDL_H
#  include <ltdl.h>
extern lt_dlhandle  __Plugin_load (const char* Name) ATTR(INIT);
# endif

#include "LDL.h"
C_BEGIN

# define DILLO_NEWIN     (1)
# define DILLO_SHARE_ERR (2)
# define DILLO_TTY       (4)

/* Bill of materials list */
typedef struct __BOM_t
{
   struct __BOM_t *q_forw, *q_back; /* List linkages */
   double      Quantity; /* The amount of each item that is needed */
   const char* Units;    /* The units that we measure the items in */
   const char* Item;     /* The item that is needed */
} __BOM_t;

typedef struct __Step_t
{
   struct __Step_t* q_forw, *q_back;
   const char* Step;
} __Step_t;

typedef struct __Seg_t
{
   struct __Seg_t* q_forw, *q_back;
   const char* Heading;    /* The heading for this segment */
} __Seg_t;

typedef enum
{
   Materials,            /* The materials (BOM) used in the project */
   Tools,
   Steps,                /* The steps to do the project */
   Recipes_imported,
   LAST_SEGMENT
} __Segs_t;


typedef struct
{
   const char* Name;       /* The name of the project */
   unsigned NSegs;         /* Number of segments this recipe has */
   __Seg_t* Segment;       /* The segments in the recipe */
} __Recipe_t;


extern void   splitc(char* Src, char** Strs, const char* Sep, int Cols);

extern pid_t  __Dillo_fork (const int* ChildIO, unsigned Flags);
extern pid_t  __Dillo_fork2(int* FileDes, unsigned Flags);
extern pid_t  __Dillo_spawn(int* Filedes, char* const ArgV[],
			    char* const Env[], unsigned Flags);
extern char* const* __ArgV_create (const char*,...);
extern char* const* __Env_create  (const char* Name, ...);
extern int          __Plugins_load(const char* Path) ATTR(INIT);
extern int          __PTY_pipe(int filedes[2]);
ssize_t _screen_write (void* SPtr, const char* buf, size_t buflen);

int timeval_diff(struct timeval* Result, const struct timeval* A,
		 const struct timeval* B);

#include <regex.h>
typedef struct
{
  regex_t Reg; /* compiled regular expression */
  char*   Hint; /* the hint that gets displayed */
} __AutoHelpEntry_t;

typedef void (*LabelProc_t) (void*, const char* Short, const char* Long);

typedef struct
{
  __AutoHelpEntry_t* Rules;
  size_t             NRules;

  __AutoHelpEntry_t* CurrentPrompt; /* Currently displayed item */

  LabelProc_t DispProc;
  void*       DispProcData;
} __AutoHelp_t;

void __AutoHelp_init(__AutoHelp_t* VPtr, __AutoHelpEntry_t* Rules, int NRules,
		     LabelProc_t DispProc, void* DispProcData) ATTR(INIT);

void __AutoHelp_text(__AutoHelp_t* VPtr, const char* Text);
int __AutoHelp_load(const char *Path, __AutoHelpEntry_t**Rules,size_t* NRules)
     ATTR(INIT);

C_END
#endif
