/* $Id: monitor.c,v 1.4 2000/07/20 00:32:53 randym Exp $
 */

/* begin the prompt process */

#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <curses.h>
#include "psh.h"
#include "platform.h"
#include "IOQ.h"

extern int run; /* indicates that the program should continue to run */
extern struct termios tt; /* the original terminal state */
static void copy_to_line  (const __CQ_t* Ptr);
static void copy_to_prompt(const __CQ_t* Ptr);

static int  HelpRun=0;
static struct timeval Disp_update, *TPtr= NULL, NextUpdate;
static char prcombuf[BUFSIZE]; /* prompt / command buffer */
static int  promptend; /* index into prcombuf that the command actually starts
			  (and the prompt ends) */

void Disp_sched(void)
{
  if (TPtr) return;
  
  /* The human eye can't see a frame rate faster than 30 frames
     per second, so we update at that rate. */
  TPtr = &Disp_update;
  Disp_update.tv_sec=0;
  Disp_update.tv_usec=1000000/30;
  if (gettimeofday(&NextUpdate,NULL))
    exit(-1);
  NextUpdate.tv_usec += 1000000/30;
  if (NextUpdate.tv_usec > 1000000)
    {
      NextUpdate.tv_usec-= 1000000;
      NextUpdate.tv_sec++;
    }
}

static void _P_write(int FD, __CQ_t* P)
{
  /* This checks to see if we are sending to the shell or display */
   if (FD == STDOUT_FILENO)
     {
        /* We are sending to the display */
	Disp_sched();
	Shell_print(P);
	return;
     }

   __CQ_write(FD, P);
}

typedef struct
{
  __CQ_t Q;
  int ReadFD, WriteFD;
  char Buffer[BLKSIZE];
} PipeBuf;


static PipeBuf ShBuf={
  /* Stuff from shell to user */
  {&ShBuf.Buffer[0], &ShBuf.Buffer[BLKSIZE],
   &ShBuf.Buffer[0], &ShBuf.Buffer[0]}},
  UsrBuf ={ /*Stuff from user to shell */
    {&UsrBuf.Buffer[0],&UsrBuf.Buffer[BLKSIZE],
     &UsrBuf.Buffer[0],&UsrBuf.Buffer[0]}};

static int lastkey; /* last key entered from the terminal */
static void Shell_proc(const __CQ_t* Ptr);
    
#ifdef HAVE_GLIB0
#include "glib.h"

static gboolean P_write(GIOChannel* Src, GIOCondition Why, gpointer Data)
{
   PipeBuf* Ptr = (PipeBuf*) Data;
   if (Why != G_IO_OUT) return 0;
   _P_write(Ptr->WriteFD, &Ptr->Q);

   /* Queue up read */
   if (Ptr->Q.Head != Ptr->Q.Tail+1 && !
       (Ptr->Q.Tail+1 == Ptr->Q.End && Ptr->Q.Head == Ptr->Q.Begin))
     ;
   return 1;
}

static gboolean P_read(GIOChannel* Src, GIOCondition Why, gpointer Data)
{
   PipeBuf* Ptr = (PipeBuf*) Data;
   if (Why != G_IO_IN) return 0;
   __CQ_read(Ptr->ReadFD, &Ptr->Q);
   /* Queue up write */
   if (Ptr->Q.Head != Ptr->Q.Tail) ;
   return 1;
}
#else

static void P_do(fd_set* R, fd_set* W, PipeBuf* P)
{
  if (FD_ISSET(P->WriteFD, W) && P->Q.Head != P->Q.Tail)
    _P_write(P->WriteFD, &P->Q);
  if (FD_ISSET(P->ReadFD, R))
    __CQ_read (P->ReadFD, &P->Q);
}

static void P_fds(fd_set* R, fd_set* W, PipeBuf* P)
{
  if (P->Q.Head != P->Q.Tail)
    FD_SET(P->WriteFD, W);
  FD_SET(P->ReadFD, R); 
}

static int Fds;

static void P_dispatch(void)
{
  fd_set readset, writeset; /* read set - from the shell or terminal */
  while (run)
   {
      int RunUpdate=0;
      FD_ZERO(&readset);
      FD_ZERO(&writeset);
      P_fds(&readset, &writeset, &UsrBuf);
      P_fds(&readset, &writeset, &ShBuf);
      
      /* wait indefinately for the user or shell to be ready */
      switch (select(Fds, &readset, &writeset, NULL, TPtr))
      {
        case 0: RunUpdate++; TPtr=NULL;break;
	case -1:
	  if (errno != EINTR) die("select failed", -2, errno);
	default:
	  /* check which descriptor has data */
	  P_do(&readset, &writeset, &ShBuf);
	  if (ShBuf.Q.Head != ShBuf.Q.Tail)
	    {
	       Shell_proc(&ShBuf.Q);
	    }
	  
	  P_do(&readset, &writeset, &UsrBuf);
	  if (UsrBuf.Q.Head != UsrBuf.Q.Tail)
	    {
	      /* note the last character typed - I know, this doesn't cover the
		 case where two characters get read, and the newline isn't the
		 last; I just don't think dealing with that is worthwhile */
	      lastkey = UsrBuf.Q.Head[__CQ_write_size(&UsrBuf.Q)-1];
	    }

	  /* Check to see if we need to update the display timer*/
	  if (TPtr)
	    {
	      struct timeval ThisMoment;
	      if (gettimeofday(&ThisMoment, NULL))
		exit(-1);
	      if (timeval_diff(TPtr, &NextUpdate, &ThisMoment) < 0)
		{
		  TPtr = NULL;
		  RunUpdate++;
		}
	    }
	  break;
      }
      
      if (RunUpdate)
	{
	   if (HelpRun)
	     {
	       HelpRun=0;
	       __AutoHelp_text(&HelpDisp, prcombuf+promptend);
	     }
	   update_panels();
	   doupdate();
	}
   }
}

#endif

static __PSH_state_t state; /* current input state - this gets tricky :) */
static int matchstate; /* whether we are matching a prompt or getting a new one */
static char linebuf[BUFSIZE]; /* input buffer, so far */
static int  lineend; /* index into linebuf that the current input ends */
static int comend; /* index into prcombuf that the command ends */

void monitor(int mfd)
{
  //GMainLoop* GPtr = g_main_new(0);

  reset_line();
  reset_prompt();
  Fds = Max(Max(STDOUT_FILENO, STDIN_FILENO), mfd)+1;
  ShBuf.ReadFD=mfd;
  ShBuf.WriteFD=STDOUT_FILENO;
  UsrBuf.ReadFD=STDIN_FILENO;
  UsrBuf.WriteFD=mfd;

  //  FD_bkgnd(STDOUT_FILENO);
  FD_bkgnd(STDIN_FILENO);
  FD_bkgnd(mfd);
  //FD_notification(FD, Callback, Data);
  P_dispatch();
}

void Shell_proc(const __CQ_t* Ptr)
{
  /* deal with shell output */
  switch (state) {
  case NOPROMPT:
    copy_to_line(Ptr); /* update the line with the new
			       characters */
    set_confirm_prompt();
    break;
  case CONFIRMPROMPT:
  case HAVEPROMPT:
    copy_to_prompt(Ptr);
    prcombuf[comend] = '\0'; /* Zero terminate it */
    HelpRun++;
    break;
  default:
    die("internal error - fell out of switch on shell read",
	-2, errno);
    break;
  }
}

/* set us into CONFIRMPROMPT state */
void set_confirm_prompt()
{
  if (matchstate && memcmp(prcombuf, linebuf, comend) == 0) {
    /* they are the same - back to HAVEPROMPT */
    bcopy(linebuf, prcombuf, lineend); /* make sure we get anything new */
    comend = lineend;
    set_have_prompt();
  }
  else {
    /* copy the line to the prompt */
    bcopy(linebuf, prcombuf, lineend);
    promptend = comend = lineend;
    state = CONFIRMPROMPT;
  }
}

/* clear the line of all data */
void reset_line()
{
  lineend = 0;
  lastkey = NOINPUT;
  HelpRun = 0;
}

/* clear the prompt and command of all data */
void reset_prompt()
{
  if (!matchstate) {
    promptend = 0;
    comend = 0; /* no prompt so far */
  }
  lastkey = NOINPUT;
  HelpRun=0;
}

static const char* SkipCRLF(const __CQ_t* Ptr);
static void _copy_to_line(const __CQ_t* Ptr, const char* Head);


/* copy characters to the line buffer, resetting if we go over a line */
void copy_to_line(const __CQ_t* Ptr)
{
   const char* Head = SkipCRLF(Ptr);

   /* carriage return or line feed, back to square one */
   if (Head != Ptr->Head) reset_line();
   _copy_to_line(Ptr, Head);
}

static const char* SkipCRLF(const __CQ_t* Ptr)
{
  const char* Head = Ptr->Head, *Head2;

  if (Ptr->Tail == Ptr->Head) return Head;

  /* Scan for new line and/or carriage return. */
  if ((Head2 = memchr(Head, CR, ___CQ_write_size(Ptr,Head)))) Head = Head2;
  if ((Head2 = memchr(Head, LF, ___CQ_write_size(Ptr,Head)))) Head = Head2;

  /* See if we should do it again */
  if (Ptr->Tail >= Head) return Head;

  if ((Head2 = memchr(Head, CR, ___CQ_write_size(Ptr,Head)))) Head = Head2;
  if ((Head2 = memchr(Head, LF, ___CQ_write_size(Ptr,Head)))) Head = Head2;
  return Head;
}
  
/* copy characters to the line buffer, resetting if we go over a line */
static void _copy_to_line(const __CQ_t* Ptr, const char* Head)
{
  for (;Head != Ptr->Tail && lineend < BUFSIZE;
       Head=(Head+1)!=Ptr->End?Head+1:Ptr->Begin)
    {
      if (strchr(IGNORECHARS, *Head)) continue;
      /* not in ignore list, proceed */
      switch (*Head)
      {
        case BS: /* backspace, back it up there */
	  if (lineend > 0) --lineend;
	  break;
        default:
	  linebuf[lineend++] = *Head;
      }
  }
}

/* copy characters to the prompt, dealing with CONFIRMPROMPT and HAVEPROMPT
   states; change states as appropriate */
void copy_to_prompt(const __CQ_t* Ptr)
{
  const char* Head;

  if (state == CONFIRMPROMPT && lastkey != NOINPUT) { /* user hit a key,
							 confirming that we
							 have finished the
							 prompt */
    set_have_prompt();
  }

   Head = SkipCRLF(Ptr);

   /* carriage return or line feed, back to square one */
   if (Head != Ptr->Head)
     {
       switch (state)
       {
          case CONFIRMPROMPT:
	    /* newline of any sort means we don't actually have a prompt,
	       it was just app lag */
	    if (!matchstate) reset_prompt();
	    break;
	  
          case HAVEPROMPT:
	    /* newline... which state do we go to? */
	    if (lastkey == CR || lastkey == LF) { /* user typed a newline, so
						     this is most likely a normal
						     end of command, so back to
						     NOPROMPT */
	      if (!matchstate) reset_prompt();
	      matchstate = 0;
	    }
	    else { /* hmmm... user didn't type a return... go to MATCHPROMPT */
	      matchstate = 1;
	      reset_prompt();
	      reset_line();
	    }
          default: break;
       }
       state = NOPROMPT;
       _copy_to_line(Ptr, Head);
       /* *must* break the loop :-) */
       return;
     }


  switch (state) {
  case CONFIRMPROMPT: /* no chance of going to MATCHPROMPT state */
    for (;Head != Ptr->Tail; Head=(Head+1)!=Ptr->End?Head+1:Ptr->Begin)
      {
	if (strchr(IGNORECHARS, *Head)) continue;
	/* not in ignore list, proceed */
	switch (*Head)
	 {
	 case BS: /* backspace, back it up */
	   if (promptend > 0 && promptend < BUFSIZE) --promptend;
	   break;
	 default:
	   if (promptend < BUFSIZE)
	     prcombuf[promptend++] = *Head;
	 }
	comend = promptend;
      }
    break;
  case HAVEPROMPT: /* chance of going to MATCHPROMPT state */
    for (;Head != Ptr->Tail; Head=(Head+1)!=Ptr->End?Head+1:Ptr->Begin)
      {
	if (strchr(IGNORECHARS, *Head)) continue;
	/* not in ignore list, proceed */
	
	switch (*Head)
	 {
	 case BS: /* back it up */
	   if (comend > promptend && promptend < BUFSIZE) --comend;
	   break;
	 default:
	   if (promptend < BUFSIZE)
	     prcombuf[comend++] = *Head;
	 }
      }
    break;
  default: break;
  }
}

void set_have_prompt()
{
  state = HAVEPROMPT;
  matchstate = 0;
}
