/* $Id: psh.c,v 1.4 2000/08/12 14:14:28 randym Exp randym $
 
   Help Window access routines
   Copyright 2000 Randall Maas (randym@acm.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


/* Test opening pipes to a shell */

#include <stdio.h>
#include <stdlib.h>
#include "platform.h"
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <termios.h>
#include <unistd.h>
#include "psh.h"
#include "../dillib/vty/vtcurses.h"

int run = 0; /* indicates that the program should continue to run */
pid_t child_pid; /* PID of the child shell; must be global so the signal
		    handlers can see it */

void die (const char *message, int code, int syserr)
{
  Err_cmdline("%s: %s\n", message, strerror(syserr));
  exit (code);
}

// --- CURSES initialization and exit ----------------------------------------
__AutoHelp_t HelpDisp;
extern __AutoHelpEntry_t* helplist;
extern int numhints;
static void Disp_init(const char* buf) ATTR(INIT);

static void Disp_init(const char* buf)
{
  __VT_label_t* Disp;
  if (!initscr())
    {
      signals_mode(NORMAL);
      exit(-1);
    }
  noecho ();
  crmode ();
  nonl   ();
  intrflush(stdscr, FALSE);

#ifdef A_COLOR
  use_default_colors();
  start_color();
  //  if (has_colors())
    //    init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLUE);
#endif /* A_COLOR */

  keypad   (stdscr, TRUE);
  raw();
  curs_set(1);
  refresh();

  Disp = __VT_label_new(buf);
  __AutoHelp_init(&HelpDisp, helplist, numhints,
		  (LabelProc_t) __VT_label_text, Disp);
}

static void End(void)
{
  __ShellWin_exit();
  __VT_label_destroy(HelpDisp.DispProcData);
  clear();
  refresh();
  signals_mode(NORMAL);
  endwin();

  Err_cmdline("session ended, goodbye.\n");
}


extern int numhints;
int main (int argc, char *argv[], char** env)
{
  const char* argv0=argv[0], *Term=getenv("TERM");
  char *shellpath;
  int filedes[2];  /* master file descriptor, pseudo tty to the slave shell */
   char buf[1024];

   /* Check that we are using ttys for in and out */
   check_stdio();

   /* now figure out what shell to use */
  if ((shellpath = getenv(PSHVAR)) == NULL) {
    if ((shellpath = getenv(SHELLVAR)) == NULL) {
      shellpath = DEFAULTSHELL;
    }
  }

  /* Preserve /our/ terminal setting */
  sprintf(buf,"TERM=%s",Term);

  argv[0]=shellpath;
  //putenv("TERM=vt100"); /* Tell our child processes that we emulate vt100's*/
  child_pid = __Dillo_spawn(filedes, argv, env,
			    DILLO_TTY|DILLO_SHARE_ERR|DILLO_NEWIN); /* now we are two */
  if (child_pid == -1) { /* failed */
    die ("unable to fork child shell process", -1, errno);
  }

  /* Restore our terminal setting */
  //putenv(buf); /* Busted somehow... causes ncurses to die ? */
   load_all_help();
   sprintf(buf,"This is [%s], running over [%s], with a %u prompt database.\n",
	  argv0, shellpath, numhints);
  Disp_init(buf);
  atexit(End);


  /* this is the parent, after a successful fork() */
  signals_mode(LIVE); /* set signal handlers */
  change_window(filedes[0]); /* change the window size correctly */

  __ShellWin_init(filedes[0]);
  monitor(filedes[0]);

  return 0;
}
