/* $Id: psh.h,v 1.4 2000/08/12 14:14:28 randym Exp randym $
 */

/* must do this since a structure using it is defined below */
#include "dillib.h"
#include <regex.h>
#include <curses.h>
#include <panel.h>
#include "circque.h"

/* the string where our database is. */
#define GLOBALHELP DATADIR "/ah-tty.conf"
#define HOMEHELP ".ah-ttyrc"

#define LIVE 1 /* specifies a "live" state */
#define NORMAL 0 /* specifies a "normal" state, i.e. no signal handlers */

#define PSHVAR "PSHELL"
#define SHELLVAR "SHELL"
#define DEFAULTSHELL "/bin/sh"

#define BLKSIZE ((size_t)1024) /* max block read at any one time */

typedef enum
{
  NOPROMPT,     /* don't have a good prompt yet */
  CONFIRMPROMPT,/* have a prompt, we think, but no shell input just yet */
  HAVEPROMPT,   /* have a prompt, now reading command line */
  MATCHPROMPT,  /* unexpected newline - matching output to the previous
		   prompt */
} __PSH_state_t;
#define NOINPUT -1 /* indicates that no characters have been input since
		      last cleared */
#define BUFSIZE 512 /* size of input line buffers etc. */

#define UPLEVEL "+" /* increase help level */
#define DOWNLEVEL "-" /* decrease help level */
#define MINLEVEL 0
#define MAXLEVEL 7

#if 0
  unsigned short helplevel; /* the help levels in which this hint will be
			       displayed */
#endif

/* characters of significance */
#define CR '\r'
#define LF '\n'
#define BS '\b'
/* characters of anti-significance - those we ignore for the purposes of
   monitoring the shell's output, such as characters that do not print on
   the screen - BEL is a good example (only one yet:) */
#define IGNORECHARS "\007"

extern WINDOW* ShellWin, *HelpWin;
extern PANEL*  ShellPanel, *HelpPanel;

/* function declarations */
void die(const char*, int, int);
void signals_mode(int);
void monitor(int mfd);
void set_confirm_prompt();
void reset_line();
void reset_prompt();
void match_prompt();
void set_no_prompt();
void set_confirm_prompt();
void set_have_prompt();
void check_stdio  (void)	ATTR(INIT);
void load_all_help(void)        ATTR(INIT);
void change_window(int fd);
void Disp_sched   (void);

void __ShellWin_init(int fd)	ATTR(INIT);
void __ShellWin_exit(void)	ATTR(INIT);
void Shell_print(__CQ_t* Ptr);

extern __AutoHelp_t HelpDisp;
